# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from thx.api.mixins import ThuxModelMixin
from thx.role.core.views import ThuxRoleViewSetMixin

from ... import models
from . import serializers, filters, permissions

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# CARRIAGE
class CarriageViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    CarriageViewSet
    -----------

    :action list: Return the carriage list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given carriage.
        `Profiles`: `superuser`, `staff`

    :action light: Return the carriage list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new carriage instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a carriage instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a carriage instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a carriage instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Carriage.objects.all()
    serializer_class = serializers.CarriageSerializer
    filter_backends = (filters.CarriageFilterBackend,)
    permission_classes = (permissions.CarriagePermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a carriage instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a carriage instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# DELIVERY
class DeliveryViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    DeliveryViewSet
    -----------

    :action list: Return the delivery list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given delivery.
        `Profiles`: `superuser`, `staff`

    :action light: Return the delivery list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new delivery instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a delivery instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a delivery instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a delivery instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Delivery.objects.all()
    serializer_class = serializers.DeliverySerializer
    filter_backends = (filters.DeliveryFilterBackend,)
    permission_classes = (permissions.DeliveryPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a delivery instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a delivery instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)
