# -*- coding: utf-8 -*-

from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _
from thx.core.models import CleanModel, DateModel, OrderedModel, StatusModel, UserModel


class Carriage(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    """
    Stores carriages:
    frw - CARRIAGE FORWARD - PORTO ASSEGNATO
    pad - CARRIAGE PAID - PORTO FRANCO
    exw - EX WORKS" - EX WORKS
    pdc - CARRIAGE PAID W/CHARGE - PORTO FRANCO CON ADDEBITO
    """
    code = models.CharField(
        max_length=3,
        primary_key=True,
        verbose_name=_("code")
    )
    name = models.CharField(
        max_length=50,
        unique=True,
        verbose_name=_("name")
    )
    slug = models.SlugField(
        max_length=50,
        unique=True,
        verbose_name=_("slug"),
    )
    description = models.TextField(
        blank=True,
        verbose_name=_("description")
    )

    class Meta:
        verbose_name = _('carriage')
        verbose_name_plural = _('carriages')
        ordering = ('ordering', 'name',)
        permissions = (
            ("list_carriage", "Can list carriage"),
            ("detail_carriage", "Can detail carriage"),
            ("enable_carriage", "Can enable carriage"),
            ("disable_carriage", "Can disable carriage"),
        )

    def __str__(self):
        return self.name


class Delivery(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    code = models.CharField(
        max_length=50,
        primary_key=True,
        verbose_name=_("code")
    )
    transport_by = models.CharField(
        max_length=3,
        choices=settings.CARRIAGE_TRANSPORTBY_DEF,
        verbose_name=_('transport by'),
    )
    carriage = models.ForeignKey(
        Carriage,
        on_delete=models.PROTECT,
        verbose_name=_('carriage'),
    )

    class Meta:
        verbose_name = _('delivery')
        verbose_name_plural = _('deliveries')
        ordering = ('ordering',)
        permissions = (
            ("list_delivery", "Can list delivery"),
            ("detail_delivery", "Can detail delivery"),
            ("enable_delivery", "Can enable delivery"),
            ("disable_delivery", "Can disable delivery"),
        )

    def __str__(self):
        return f"{self.get_transport_by_display()} - {self.carriage}"
